@@ -533,7 +533,7 @@ int MY_FAST_CALL LZMA_DECODE_REAL(CLzmaDec *p, SizeT limit, const Byte *bufLimit
       {
         SizeT rem;
         unsigned curLen;
-        SizeT pos;
+        SizeT pos = 0;
 
         if ((rem = limit - dicPos) == 0)
         {
@@ -542,7 +542,8 @@ int MY_FAST_CALL LZMA_DECODE_REAL(CLzmaDec *p, SizeT limit, const Byte *bufLimit
         }
 
         curLen = ((rem < len) ? (unsigned)rem : len);
-        pos = dicPos - rep0 + (dicPos < rep0 ? dicBufSize : 0);
+        if (rep0 <= dicPos)
+         pos = dicPos - rep0 + (dicPos < rep0 ? dicBufSize : 0);
 
         processedPos += (UInt32)curLen;
 
