@@ -121,7 +121,7 @@ int cthd_sysfs_cdev_rapl::get_max_state() {
 
 int cthd_sysfs_cdev_rapl::rapl_sysfs_valid()
 {
-	std::stringstream temp_str;
+	std::ostringstream temp_str;
 	int found_long_term = 0;
 	int i;
 
@@ -151,7 +151,7 @@ int cthd_sysfs_cdev_rapl::rapl_sysfs_valid()
 	temp_str.str(std::string());
 	temp_str << "constraint_" << constraint_index << "_power_limit_uw";
 	if (!cdev_sysfs.exists(temp_str.str())) {
-		thd_log_debug("powercap RAPL no  power limit uw %s \n",
+		thd_log_debug("powercap RAPL no  power limit uw %s\n",
 				temp_str.str().c_str());
 		return THD_ERROR;
 	}
@@ -159,7 +159,7 @@ int cthd_sysfs_cdev_rapl::rapl_sysfs_valid()
 	temp_str.str(std::string());
 	temp_str << "constraint_" << constraint_index << "_time_window_us";
 	if (!cdev_sysfs.exists(temp_str.str())) {
-		thd_log_info("powercap RAPL no time_window_us %s \n",
+		thd_log_info("powercap RAPL no time_window_us %s\n",
 				temp_str.str().c_str());
 		return THD_ERROR;
 	}
@@ -169,7 +169,7 @@ int cthd_sysfs_cdev_rapl::rapl_sysfs_valid()
 
 int cthd_sysfs_cdev_rapl::rapl_read_pl1_max()
 {
-	std::stringstream temp_power_str;
+	std::ostringstream temp_power_str;
 	int current_pl1_max;
 
 	temp_power_str << "constraint_" << constraint_index << "_max_power_uw";
@@ -182,7 +182,7 @@ int cthd_sysfs_cdev_rapl::rapl_read_pl1_max()
 
 int cthd_sysfs_cdev_rapl::rapl_read_pl1()
 {
-	std::stringstream temp_power_str;
+	std::ostringstream temp_power_str;
 	int current_pl1;
 
 	temp_power_str << "constraint_" << constraint_index << "_power_limit_uw";
@@ -195,14 +195,14 @@ int cthd_sysfs_cdev_rapl::rapl_read_pl1()
 
 int cthd_sysfs_cdev_rapl::rapl_update_pl1(int pl1)
 {
-	std::stringstream temp_power_str;
+	std::ostringstream temp_power_str;
 	int ret;
 
 	temp_power_str << "constraint_" << constraint_index << "_power_limit_uw";
 	ret = cdev_sysfs.write(temp_power_str.str(), pl1);
 	if (ret <= 0) {
 		thd_log_info(
-				"pkg_power: powercap RAPL max power limit failed to write %d \n",
+				"pkg_power: powercap RAPL max power limit failed to write %d\n",
 				pl1);
 		return ret;
 	}
@@ -212,7 +212,7 @@ int cthd_sysfs_cdev_rapl::rapl_update_pl1(int pl1)
 
 int cthd_sysfs_cdev_rapl::rapl_read_pl2()
 {
-	std::stringstream temp_power_str;
+	std::ostringstream temp_power_str;
 	int current_pl2;
 
 	temp_power_str << "constraint_" << pl2_index << "_power_limit_uw";
@@ -225,7 +225,7 @@ int cthd_sysfs_cdev_rapl::rapl_read_pl2()
 
 int cthd_sysfs_cdev_rapl::rapl_update_pl2(int pl2)
 {
-	std::stringstream temp_power_str;
+	std::ostringstream temp_power_str;
 	int ret;
 
 	if (pl2_index == -1) {
@@ -236,7 +236,7 @@ int cthd_sysfs_cdev_rapl::rapl_update_pl2(int pl2)
 	ret = cdev_sysfs.write(temp_power_str.str(), pl2);
 	if (ret <= 0) {
 		thd_log_info(
-				"pkg_power: powercap RAPL max power limit failed to write PL2 %d \n",
+				"pkg_power: powercap RAPL max power limit failed to write PL2 %d\n",
 				pl2);
 		return ret;
 	}
@@ -246,7 +246,7 @@ int cthd_sysfs_cdev_rapl::rapl_update_pl2(int pl2)
 
 int cthd_sysfs_cdev_rapl::rapl_read_time_window()
 {
-	std::stringstream temp_time_str;
+	std::ostringstream temp_time_str;
 	int tm_window;
 
 	temp_time_str << "constraint_" << constraint_index << "_time_window_us";
@@ -259,13 +259,13 @@ int cthd_sysfs_cdev_rapl::rapl_read_time_window()
 
 int cthd_sysfs_cdev_rapl::rapl_update_time_window(int time_window)
 {
-	std::stringstream temp_time_str;
+	std::ostringstream temp_time_str;
 
 	temp_time_str << "constraint_" << constraint_index << "_time_window_us";
 
 	if (cdev_sysfs.write(temp_time_str.str(), time_window) <= 0) {
 		thd_log_info(
-				"pkg_power: powercap RAPL time window failed to write %d \n",
+				"pkg_power: powercap RAPL time window failed to write %d\n",
 				time_window);
 		return THD_ERROR;
 	}
@@ -275,13 +275,13 @@ int cthd_sysfs_cdev_rapl::rapl_update_time_window(int time_window)
 
 int cthd_sysfs_cdev_rapl::rapl_update_pl2_time_window(int time_window)
 {
-	std::stringstream temp_time_str;
+	std::ostringstream temp_time_str;
 
 	temp_time_str << "constraint_" << pl2_index << "_time_window_us";
 
 	if (cdev_sysfs.write(temp_time_str.str(), time_window) <= 0) {
 		thd_log_info(
-				"pkg_power: powercap RAPL time window failed to write %d \n",
+				"pkg_power: powercap RAPL time window failed to write %d\n",
 				time_window);
 		return THD_ERROR;
 	}
@@ -291,12 +291,12 @@ int cthd_sysfs_cdev_rapl::rapl_update_pl2_time_window(int time_window)
 
 int cthd_sysfs_cdev_rapl::rapl_update_enable_status(int enable)
 {
-	std::stringstream temp_str;
+	std::ostringstream temp_str;
 
 	temp_str << "enabled";
 	if (cdev_sysfs.write(temp_str.str(), enable) <= 0) {
 		thd_log_info(
-				"pkg_power: powercap RAPL enable failed to write %d \n",
+				"pkg_power: powercap RAPL enable failed to write %d\n",
 				enable);
 		return THD_ERROR;
 	}
@@ -306,7 +306,7 @@ int cthd_sysfs_cdev_rapl::rapl_update_enable_status(int enable)
 
 int cthd_sysfs_cdev_rapl::rapl_read_enable_status()
 {
-	std::stringstream temp_str;
+	std::ostringstream temp_str;
 	int enable;
 
 	temp_str << "enabled";
@@ -417,10 +417,10 @@ int cthd_sysfs_cdev_rapl::update() {
 			int ret = cdev_sysfs.read("name", domain_name);
 
 			if (!ret)
-				thd_log_info("%s:powercap RAPL invalid max power limit range \n",
+				thd_log_info("%s:powercap RAPL invalid max power limit range\n",
 						domain_name.c_str());
 
-			thd_log_info("Calculate dynamically phy_max \n");
+			thd_log_info("Calculate dynamically phy_max\n");
 
 			power_on_constraint_0_pwr = rapl_read_pl1();
 			thd_log_debug("power_on_constraint_0_pwr %d\n",
@@ -449,7 +449,7 @@ int cthd_sysfs_cdev_rapl::update() {
 					constraint_phy_max, phy_max);
 			phy_max = constraint_phy_max;
 		}
-		thd_log_info("powercap RAPL max power limit range %d \n", phy_max);
+		thd_log_info("powercap RAPL max power limit range %d\n", phy_max);
 
 		set_inc_dec_value(-phy_max * (float) rapl_power_dec_percent / 100);
 		min_state = phy_max;
