@@ -33,7 +33,7 @@
 int cthd_engine_adaptive::install_passive(struct psv *psv) {
 	std::string psv_zone;
 
-	size_t pos = psv->target.find_last_of(".");
+	size_t pos = psv->target.find_last_of('.');
 	if (pos == std::string::npos)
 		psv_zone = psv->target;
 	else
@@ -64,7 +64,7 @@ int cthd_engine_adaptive::install_passive(struct psv *psv) {
 	}
 
 	std::string psv_cdev;
-	pos = psv->source.find_last_of(".");
+	pos = psv->source.find_last_of('.');
 	if (pos == std::string::npos)
 		psv_cdev = psv->source;
 	else
@@ -122,12 +122,12 @@ int cthd_engine_adaptive::install_passive(struct psv *psv) {
 	return 0;
 }
 
-void cthd_engine_adaptive::set_trip(std::string target, std::string argument) {
+void cthd_engine_adaptive::set_trip(const std::string& target, const std::string& argument) {
 	std::string psv_zone;
 	float float_temp = stof(argument, NULL);
 	int temp = (int) (float_temp * 1000);
 
-	size_t pos = target.find_last_of(".");
+	size_t pos = target.find_last_of('.');
 
 	if (pos == std::string::npos)
 		psv_zone = target;
@@ -164,7 +164,7 @@ void cthd_engine_adaptive::psvt_consolidate() {
 	 * If there is a next trip after MAX for a target, then choose a temperature limit in the middle
 	 */
 	for (unsigned int i = 0; i < zones.size(); ++i) {
-		cthd_zone *zone = zones[i];
+		cthd_zone *zone = zones[i].get();
 		unsigned int count = zone->get_trip_count();
 
 		// Special case for handling a single trip which has a defined
@@ -249,7 +249,7 @@ void cthd_engine_adaptive::psvt_consolidate() {
 int cthd_engine_adaptive::install_itmt(struct itmt_entry *itmt_entry) {
 	std::string itmt_zone;
 
-	size_t pos = itmt_entry->target.find_last_of(".");
+	size_t pos = itmt_entry->target.find_last_of('.');
 	if (pos == std::string::npos)
 		itmt_zone = itmt_entry->target;
 	else
@@ -362,10 +362,10 @@ int cthd_engine_adaptive::set_itmt_target(struct adaptive_target &target) {
 
 	if (!int3400_installed) {
 		for (unsigned int i = 0; i < zones.size(); ++i) {
-			cthd_zone *_zone = zones[i];
+			cthd_zone *_zone = zones[i].get();
 
 			// This is only for debug to plot power, so keep
-			if (_zone->get_zone_type() == "rapl_pkg_power")
+			if (_zone->get_zone_type() == "rapl_pkg_power" || _zone->get_zone_type() == "power_floor")
 				continue;
 
 			_zone->zone_reset(1);
@@ -386,7 +386,7 @@ int cthd_engine_adaptive::set_itmt_target(struct adaptive_target &target) {
 
 void cthd_engine_adaptive::set_int3400_target(struct adaptive_target &target) {
 
-	if (target.code == "ITMT") {
+	if (target.code == "ITMT" || target.code == "ITMT3") {
 		if (set_itmt_target(target) == THD_SUCCESS) {
 			int3400_installed = 1;
 		}
@@ -404,10 +404,10 @@ void cthd_engine_adaptive::set_int3400_target(struct adaptive_target &target) {
 
 		if (!int3400_installed) {
 			for (unsigned int i = 0; i < zones.size(); ++i) {
-				cthd_zone *_zone = zones[i];
+				cthd_zone *_zone = zones[i].get();
 
 				// This is only for debug to plot power, so keep
-				if (_zone->get_zone_type() == "rapl_pkg_power")
+				if (_zone->get_zone_type() == "rapl_pkg_power" || _zone->get_zone_type() == "power_floor")
 					continue;
 
 				_zone->zone_reset(1);
@@ -453,7 +453,7 @@ void cthd_engine_adaptive::install_passive_default() {
 	thd_log_info("IETM_D0 processed\n");
 
 	for (unsigned int i = 0; i < zones.size(); ++i) {
-		cthd_zone *_zone = zones[i];
+		cthd_zone *_zone = zones[i].get();
 		_zone->zone_reset(1);
 		_zone->trip_delete_all();
 
@@ -487,7 +487,7 @@ void cthd_engine_adaptive::execute_target(struct adaptive_target &target) {
 
 	thd_log_info("Target Name:%s\n", target.name.c_str());
 
-	size_t pos = target.participant.find_last_of(".");
+	size_t pos = target.participant.find_last_of('.');
 	if (pos == std::string::npos)
 		name = target.participant;
 	else
@@ -618,7 +618,6 @@ int cthd_engine_adaptive::set_int3400_base_path()
 int cthd_engine_adaptive::thd_engine_init(bool ignore_cpuid_check,
 		bool adaptive) {
 	csys_fs sysfs("");
-	char *buf;
 	size_t size;
 	int res;
 
@@ -651,6 +650,10 @@ int cthd_engine_adaptive::thd_engine_init(bool ignore_cpuid_check,
 		int3400_base_path = "/sys/bus/platform/devices/INTC1042:00/";
 	} else if (sysfs.exists("/sys/bus/platform/devices/INTC1068:00")) {
 		int3400_base_path = "/sys/bus/platform/devices/INTC1068:00/";
+	} else if (sysfs.exists("/sys/bus/platform/devices/INTC10D4:00")) {
+		int3400_base_path = "/sys/bus/platform/devices/INTC10D4:00/";
+	} else if (sysfs.exists("/sys/bus/platform/devices/INTC10FC:00")) {
+		int3400_base_path = "/sys/bus/platform/devices/INTC10FC:00/";
 	} else {
 		if (set_int3400_base_path() != THD_SUCCESS)
 			return THD_ERROR;
@@ -668,20 +671,17 @@ int cthd_engine_adaptive::thd_engine_init(bool ignore_cpuid_check,
 		return THD_ERROR;
 	}
 
-	buf = new char[size];
+	std::unique_ptr<char[]> buf(new char[size]);
 	if (!buf) {
 		thd_log_error("Unable to allocate memory for GDDV");
 		return THD_FATAL_ERROR;
 	}
 
-	if (sysfs.read(int3400_base_path + "data_vault", buf, size) < int(size)) {
+	if (sysfs.read(int3400_base_path + "data_vault", buf.get(), size) < int(size)) {
 		thd_log_debug("Unable to read GDDV data vault\n");
-		delete[] buf;
 		return THD_FATAL_ERROR;
 	}
 
-	delete[] buf;
-
 	res = gddv.gddv_init(int3400_base_path);
 	if (res != THD_SUCCESS) {
 		return res;
