@@ -36,15 +36,13 @@ int cthd_zone_dynamic::read_trip_points() {
 
 	cthd_sensor *sensor = thd_engine->search_sensor(sensor_name);
 	if (!sensor) {
-		thd_log_warn("dynamic sensor: invalid sensor type \n");
+		thd_log_warn("dynamic sensor: invalid sensor type\n");
 		return THD_ERROR;
 	}
 	thd_log_info("XX index = %d\n", index);
 	cthd_trip_point trip_pt(0, trip_type, trip_temp, 0, index,
 			sensor->get_index());
 
-	trip_points.push_back(trip_pt);
-
 	cthd_cdev *cdev = thd_engine->search_cdev(cdev_name);
 	if (cdev) {
 		trip_pt.thd_trip_point_add_cdev(*cdev, cthd_trip_point::default_influence);
@@ -52,6 +50,8 @@ int cthd_zone_dynamic::read_trip_points() {
 	} else
 		return THD_ERROR;
 
+	trip_points.push_back(std::move(trip_pt));
+
 	return THD_SUCCESS;
 }
 
@@ -63,7 +63,7 @@ int cthd_zone_dynamic::zone_bind_sensors() {
 
 	cthd_sensor *sensor = thd_engine->search_sensor(sensor_name);
 	if (!sensor) {
-		thd_log_warn("dynamic sensor: invalid sensor type \n");
+		thd_log_warn("dynamic sensor: invalid sensor type\n");
 		return THD_ERROR;
 	}
 	bind_sensor(sensor);
